<?php
/**
 * Cloud Plugin: shows a cloud of the most frequently used words
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Esther Brunner <wikidesign@gmail.com>
 */
// must be run within Dokuwiki
if(!defined('DOKU_INC')) die();
 
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
 
/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_cloud extends DokuWiki_Syntax_Plugin {

  /**
   * return some info
   */
  function getInfo(){
    return array(
      'author' => 'Esther Brunner',
      'email'  => 'wikidesign@gmail.com',
      'date'   => '2006-12-02',
      'name'   => 'Cloud Plugin',
      'desc'   => 'displays the most used words in a word cloud',
      'url'    => 'http://www.wikidesign.ch/en/plugin/cloud/start',
    );
  }

  function getType(){ return 'substition'; }
  function getPType(){ return 'block'; }
  function getSort(){ return 98; }
  
  function connectTo($mode) {
    $this->Lexer->addSpecialPattern('~~CLOUD.*?~~', $mode, 'plugin_cloud');
  }

  /**
   * Handle the match
   */
  function handle($match, $state, $pos, &$handler){
    $num = substr($match, 8, -2);  // strip markup
    return $num;
  }            

  /**
   * Create output
   */
  function render($mode, &$renderer, $data) {
    if ($mode == 'xhtml'){
      
      if (!is_numeric($data)) $num = 50;
      else $num = $data;
          
      $cloud = $this->_getCloud($num, $min, $max);
      if (!$cloud) return true;
      $delta = ($max-$min)/8;
      
      // prevent caching to ensure the included pages are always fresh
      // $renderer->info['cache'] = false;
      
      // and render the cloud
      $renderer->doc .= '<div id="cloud">';
      foreach ($cloud[0] as $word => $size){
        if ($size < $min+round($delta)) $class = 'cloud1';
        elseif ($size < $min+round(2*$delta)) $class = 'cloud2';
        elseif ($size < $min+round(3*$delta)) $class = 'cloud3';
        elseif ($size < $min+round(5*$delta)) $class = 'cloud4';
        else $class = 'cloud5';
        $renderer->doc .= '<a href="'.wl($word, 'do=search').'" class="'.$class.'">'.$word.'</a> ';
      }
      $renderer->doc .= '</div>';
      return true;
    }
    return false;
  }
  
  /**
   * Returns the sorted cloud array
   */
  function _getCloud($num){
    global $conf;
  
    // load stopwords
    $swfile   = DOKU_INC.'inc/lang/'.$conf['lang'].'/stopwords.txt';
    if (@file_exists($swfile)) $stopwords = file($swfile);
    else $stopwords = array();
    
    // load extra local stopwords
    $swfile = DOKU_CONF.'stopwords.txt';
    if (@file_exists($swfile)) $stopwords = array_merge($stopwords, file($swfile));
    
    $cloud = array();
  
    if (@file_exists($conf['indexdir'].'/page.idx')){ // new word-lenght based index
      require_once(DOKU_INC.'inc/indexer.php');
      
      $n = 2; // minimum word length
      $lengths = idx_indexLengths($n);
      foreach ($lengths as $len){
        $idx      = idx_getIndex('i', $len);
        $word_idx = idx_getIndex('w', $len);
        
        $this->_addToCloud($cloud, $idx, $word_idx, $stopwords);
      }
    
    } else {                                          // old index
      $idx      = file($conf['cachedir'].'/index.idx');
      $word_idx = file($conf['cachedir'].'/word.idx');
      
      $this->_addToCloud($cloud, $idx, $word_idx, $stopwords);
    }
    
    if (count($cloud) == 0) return false;
    
    // sort by frequency, then alphabetically
    arsort($cloud);
    $cloud = array_chunk($cloud, $num, true);
    $max = current($cloud[0]);
    $min = end($cloud[0]);
    $delta = ($max-$min)/8;
    ksort($cloud[0]);
    
    return $cloud;
  }
  
  /**
   * Adds all words in given index as $word => $freq to $cloud array
   */
  function _addToCloud(&$cloud, $idx, $word_idx, &$stopwords){
    $wcount   = count($word_idx);
    
    // collect the frequency of the words
    for ($i = 0; $i < $wcount; $i++){
      $key = trim($word_idx[$i]);
      if (!is_int(array_search("$key\n", $stopwords))){
        $cloud[$key] = count(explode(":", $idx[$i]));
      }
    }
  }
   
}
 
//Setup VIM: ex: et ts=4 enc=utf-8 :