<?php
/**
 * Embed image logos
 * 
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Andreas Gohr <andi@splitbrain.org>
 * @author     Joe Lapp <joe.lapp@pobox.com>
 * @author     Dave Doyle <davedoyle.canadalawbook.ca>
 * @author     Robin Gareus <robin@gareus.org>
 */

if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
require_once(DOKU_INC.'inc/search.php');
require_once(DOKU_INC.'inc/JpegMeta.php');

class syntax_plugin_logos extends DokuWiki_Syntax_Plugin {
    /**
     * return some info
     */
    function getInfo(){
        return array(
            'author' => 'Robin Gareus',
            'email'  => 'robin@gareus.org',
            'date'   => '2005-10-16',
            'name'   => 'Fancy Gallery Plugin',
            'desc'   => 'Creates a logo image tables from meta-data, based on the gallery plugin.',
            'url'    => 'http://mir.dnsalias.org/oss:dokuwiki:logos',
        );
    }

    /**
     * What kind of syntax are we?
     */
    function getType(){
        return 'substition';
    }
   
    /**
     * What about paragraphs?
     */
    function getPType(){
        return 'block';
    }

    /**
     * Where to sort in?
     */ 
    function getSort(){
        return 301;
    }


    /**
     * Connect pattern to lexer
     */
    function connectTo($mode) {
        $this->Lexer->addSpecialPattern('\{\{logos>[^}]*\}\}',$mode,'plugin_logos');
        $this->Lexer->addSpecialPattern('\{\{screenshots>[^}]*\}\}',$mode,'plugin_logos');
        $this->Lexer->addSpecialPattern('\{\{medialist>[^}]*\}\}',$mode,'plugin_logos');
    }


    /**
     * Handle the match
     */
    function handle($match, $state, $pos, &$handler){
        $data = array();

        $data['pattern'] = "logo";
        if (!strncasecmp($match,"{{logos>",8) )
                $match = substr($match,8,-2); //strip markup from start and end
        else if (!strncasecmp($match,"{{medialist>",12) ) {
                $data['media'] = true;
                $match = substr($match,12,-2); //strip markup from start and end
        } else {
                $data['pattern'] = "screenshot";
                $match = substr($match,14,-2); //strip markup from start and end
        }

        //handle params
        list($ns,$params) = explode('?',$match,2);

        //namespace
        $data['ns'] = $ns;

        //max thumb dimensions
        if(preg_match('/\b(\d+)x(\d+)\b/i',$params,$match)){
            $data['w'] = $match[1];
            $data['h'] = $match[2];
        }else{
            $data['w'] = 120;
            $data['h'] = 120;
        }
        
        //number of images per row
        if(preg_match('/\b(\d+)\b/i',$params,$match)){
            $data['cols'] = $match[1];
        }else{
            $data['cols'] = 5;
        }

        //show the filename
        if(preg_match('/\bshowname\b/i',$params,$match)) {
            $data['showname'] = true;
        }else{
            $data['showname'] = false;
        }

        return $data;
    }

    /**
     * Create output
     */
    function render($mode, &$renderer, $data) {
        if($mode == 'xhtml'){
            $renderer->doc .= $this->_logos($data);
            return true;
        }
        return false;
    }

    /**
     * Does the logos formatting
     */
    function _logos($data){
        global $conf;
        global $lang;
        $ret = '';

        //use the search to get all files
        $ns = cleanID($data['ns']);
        $dir = utf8_encodeFN(str_replace(':','/',$ns));
        $files = array();
        if ($data[media]) 
          search($files,$conf['mediadir'],'search_allmedia',array(),$dir);
        else
          search($files,$conf['datadir'],'search_logos',$data,$dir);

        //anything found?
        if(!count($data)){
            $ret .= '<div class="nothing">'.$lang['nothingfound'].'</div>';
            return $ret;
        }

        // build logos
        if($data['cols'] > 0){ // format as table
            $ret .= '<table class="logos">';
            $i = 0;
            foreach($files as $img){

                if(!$img['isimg']) continue;

                if($i == 0){
                    $ret .= '<tr>';
                }

                $ret .= '<td> '; 
                $ret .= $this->_image($img,$data);
                $ret .= $this->_showname($img,$data);
                $ret .= '</td>';

                $i++;

                if($i == $data['cols']){
                    $ret .= '</tr>';
                    $i = 0;
                }
            }
            // add remaining empty cells
            for(;$i < $data['cols']; $i++){
                $ret .= '<td></td>';
            }
            
            $ret .= '</table>';
        }else{ // format as div sequence
            $ret .= '<div class="logos">';
            
            foreach($files as $img){
                if(!$img['isimg']) continue;
                
                $ret .= '<div>';
                $ret .= $this->_image($img,$data);
                $ret .= $this->_showname($img,$data);
                $ret .= '</div> ';
            }
            
            $ret .= '<br style="clear:both" /></div>';
        }

        return $ret;
    }

    /**
     * Defines how a thumbnail should look like
     */
    function _image($img,$data){
        global $ID;

        $w = $data['w'];
        $h = $data['h'];

        //prepare image src and link
        if ($data[media]) {
        # $src = ml($img['id'],array('w'=>$w,'h'=>$h));
          $src = ml($img['id'],array('w'=>$w));
          $lnk = ml($img['id'],array(),false);
        } else {
          $src = ml($img['src'],array('w'=>$w));
          $lnk = wl($img['id'],array(),false);
        }

        //prepare attributes
        $p           = array();
        $p['width']  = $w;
     #  $p['height'] = $h;
        $p['border'] = 0;
        $p['title']  = $img['id'];
        $p['alt']    = $img['id']; 
        $p['class']  = 'tn';
        $att = buildAttributes($p);

        // prepare output
        $ret  = '';
        $ret .= '<a href="'.$lnk.'">';
        $ret .= '<img src="'.$src.'" '.$att.' />';
        $ret .= '</a>';
        return $ret;
    }


    /**
     * Defines how a filename + link should look
     */
    function _showname($img,$data){
        global $ID;
 
        if(!$data['showname']) { return ''; }
 
        //prepare link
      # $lnk = ml($img['id'],array('id'=>$ID),false);
        if ($data[media]) 
          $lnk = ml($img['id'],array(),false);
        else
          $lnk = wl($img['id'],array(),false);
 
        // prepare output
        $ret  = '';
        $ret .= '<br /><a href="'.$lnk.'">';
        $ret .= htmlspecialchars($img['id']);
        $ret .= '</a>';
        return $ret;
    }

}

/**
 * Build the browsable recursive meta index of pages / dirs
 * TODO move this into Plugin: lib/plugins/logos
 *
 * $opts['ns'] is the current namespace
 *
 * @author  Andreas Gohr <andi@splitbrain.org>
 * @author  Robin Gareus <robin@gareus.org>
 */
function search_logos(&$data,$base,$file,$type,$lvl,$opts) {
  $metapattern=$opts['pattern'];
  $return = true;
  $item = array();
  if($type == 'd' && !preg_match('#^'.$file.'(/|$)#','/'.$opts['ns'])){
    return true;
  }elseif($type == 'f' && ($opts['nofiles'] || !preg_match('#\.txt$#',$file))){
    return false;
  }

  $id = pathID($file);

  //check hidden
  if(isHiddenPage($id)){
    return false;
  }

  //check ACL
  if($type=='f' && auth_quickaclcheck($id) < AUTH_READ){
    return false;
  }
  $meta= p_get_metadata($id);
  if (!$meta[$metapattern]) return false;

  $data[]=array( 'id'    => $id,
                 'type'  => $type,
                 'level' => $lvl,
                 'isimg' => true,
                 'src'   => $meta[$metapattern],
                 'url'   => $meta[url],
            #    'file'  => basename($file),
                 'meta'  => $meta,
            #    'size'  => filesize($base.'/'.$file),
            #    'writable'  => is_writable($base.'/'.$file),
                 'open'  => $return );

  return $return;
}

/**
 * List all mediafiles in a namespace
 *
 * @author  Andreas Gohr <andi@splitbrain.org>
 * @author  Robin Gareus <robin@gareus.org>
 */
function search_allmedia(&$data,$base,$file,$type,$lvl,$opts){
  if($type == 'd') return true;

  $info         = array();
  $info['id']   = pathID($file,true);

  //check ACL for namespace (we have no ACL for mediafiles)
  if(auth_quickaclcheck(getNS($info['id']).':*') < AUTH_READ){
    return false;
  }

  $info['file'] = basename($file);
  $info['size'] = filesize($base.'/'.$file);
  $info['writable'] = is_writable($base.'/'.$file);
  if(preg_match("/\.(jpe?g|gif|png)$/",$file)){
    $info['isimg'] = true;
    require_once(DOKU_INC.'inc/JpegMeta.php');
    $info['meta']  = new JpegMeta($base.'/'.$file);
  }else{
    $info['isimg'] = false;
  }
  $data[] = $info;

  return false;
}

//Setup VIM: ex: et ts=4 enc=utf-8 :
