<?php
/**
 * Folded text Plugin: enables folded text font size with syntax +-+- text +-+-
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Fabian van-de-l_Isle <webmaster [at] lajzar [dot] co [dot] uk>
 * @author     Christopher Smith <chris@jalakai.co.uk>
 * @author     Esther Brunner <esther@kaffeehaus.ch>
 * @author     Robin Gareus <robin@gareus.org>
 */

if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');

// maintain a global count of the number of folded elements in the page, 
// this allows each to be uniquely identified
global $plugin_menufold_count;
if (!isset($plugin_menufold_count)) $plugin_menufold_count = 0;

// global used to indicate that the localised folder link title tooltips 
// strings have been written out
global $plugin_menufold_strings_set;
if (!isset($plugin_menufold_string_set)) $plugin_menufold_string_set = false;

/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_menufold_div extends DokuWiki_Syntax_Plugin {

   /**
    * return some info
    */
    function getInfo(){
        return array(
            'author' => 'Robin Gareus',
            'email'  => 'robin@gareus.org',
            'date'   => '2007-01-18',
            'name'   => 'Folded Menu',
            'desc'   => 'Enable blocks of folded text. 
                         Syntax: +-+-title|folded content+-+-',
            'url'    => 'http://wiki.splitbrain.org/plugin:menufolded',
        );
    }

    function getType(){ return 'container'; }
    function getPType() { return 'block'; }
    function getAllowedTypes() { return array('container','substition','protected','disabled','formatting'); }
    function getSort(){ return 404; }
    function connectTo($mode) { $this->Lexer->addEntryPattern('\+\-\+\-.*?\|(?=.*\+\-\+\-)',$mode,'plugin_menufold_div'); }
    function postConnect() { $this->Lexer->addExitPattern('\+\-\+\-','plugin_menufold_div'); }

   /**
    * Handle the match
    */
    function handle($match, $state, $pos, &$handler){
        if ($state == DOKU_LEXER_ENTER){
            $match = trim(substr($match,4,-1)); // strip markup
            $handler->status['plugin_menufold'] = true;

            if (!$this->register_hook) {

              global $EVENT_HANDLER;
              $EVENT_HANDLER->register_hook('PARSER_HANDLER_DONE','BEFORE', $this, 'add_writestrings');

              $this->register_hook = true;
            }
        }
        return array($state, $match);
    }

   /**
    * Create output
    */
    function render($mode, &$renderer, $data) {
        global $plugin_menufold_count;

        list($state, $cdata) = $data;

        if($mode == 'xhtml') {
            switch ($state){
              case DOKU_LEXER_ENTER:
                $plugin_menufold_count++;
                $divclass=" hidden";
                $aclass="";
                $lastmenu = -1;
                if (preg_match("/^.*#menufold_([0-9]+)$/", $_REQUEST['menufold'], $matches)) $lastmenu=$matches[1];
                if ($lastmenu == $plugin_menufold_count) {$divclass=""; $aclass=" open"; }
                $renderer->doc .= '<p><a class="menufolder'.$aclass.'" href="#menufold_'.$plugin_menufold_count.'">';

                if ($cdata)
                    $renderer->doc .= ' '.$renderer->_xmlEntities($cdata);

                $renderer->doc .= '</a></p><div class="menufold'.$divclass.'" id="menufold_'.$plugin_menufold_count.'">';
                break;

              case DOKU_LEXER_UNMATCHED:
                $renderer->doc .= $renderer->_xmlEntities($cdata);
                break;

              case DOKU_LEXER_EXIT:
                $renderer->doc .= '</div>';
                break;

              case 'WRITE_STRINGS' :
                if (!$plugin_menufold_strings_set) {

                  $hide = $this->getConf('hide') ? $this->getConf('hide') : $this->getLang('hide');
                  $reveal = $this->getConf('reveal') ? $this->getConf('reveal') : $this->getLang('reveal');

                  $renderer->doc .= '<span id="menufold_reveal" style="display:none;"><!-- '.hsc($reveal).' --></span>';
                  $renderer->doc .= '<span id="menufold_hide" style="display:none;"><!-- '.hsc($hide).' --></span>';

                  $plugin_menufold_strings_set = true;
                }
            }
            return true;
        }
        return false;
    }

    function add_writestrings(&$event, $param) {

      if (isset($event->plugin_menufold)) return;

      // make sure the event is being generated for the handler instance we expect
      $handler =& $event->data;
      if (empty($handler->status['plugin_menufold'])) return;

      // add WRITE_STRINGS instruction to the end of the instruction list
      $last_call = end($handler->calls);
      array_push($handler->calls, array('plugin', array('menufold_span', array('WRITE_STRINGS',0), DOKU_LEXER_MATCHED), $last_call[2]));

      // prevent multiple handling of this event by folded plugin components
      $event->plugin_menufold = true;
    }
}
