<?php
/**
 * render v-cards from meta-tags
 * 
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Andreas Gohr <andi@splitbrain.org>
 * @author     Robin Gareus <robin@gareus.org>
 */

if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
require_once(DOKU_INC.'inc/search.php');
require_once(DOKU_INC.'inc/JpegMeta.php');

class syntax_plugin_metacard extends DokuWiki_Syntax_Plugin {
    /**
     * return some info
     */
    function getInfo(){
        return array(
            'author' => 'Robin Gareus',
            'email'  => 'robin@gareus.org',
            'date'   => '2007-01-13',
            'name'   => 'Meta Tag Render Plugin',
            'desc'   => 'Creates v-cards from meta-data.',
            'url'    => 'http://mir.dnsalias.org/oss:dokuwiki:metacard',
        );
    }

    /**
     * What kind of syntax are we?
     */
    function getType(){
        return 'substition';
    }
   
    /**
     * What about paragraphs?
     */
    function getPType(){
        return 'block';
    }

    /**
     * Where to sort in?
     */ 
    function getSort(){
        return 301;
    }


    /**
     * Connect pattern to lexer
     */
    function connectTo($mode) {
        $this->Lexer->addSpecialPattern('\{\{metacard>[^}]*\}\}',$mode,'plugin_metacard');
    }


    /**
     * Handle the match
     */
    function handle($match, $state, $pos, &$handler){
        $data = array();

        $data['pattern'] = "logo";
        if (!strncasecmp($match,"{{metacard>",11) ) {
                $match = substr($match,11,-2); //strip markup from start and end
    #    } else {
    #            $data['pattern'] = "screenshot";
    #            $match = substr($match,11,-2); //strip markup from start and end
        }

        //handle params
        list($ns,$params) = explode('?',$match,2);

        //namespace
        $data['ns'] = $ns;

        //max thumb dimensions
        if(preg_match('/\b(\d+)x(\d+)\b/i',$params,$match)){
            $data['w'] = $match[1];
            $data['h'] = $match[2];
        }else{
            $data['w'] = 120;
            $data['h'] = 120;
        }
        
        //number of images per row
        if(preg_match('/\b(\d+)\b/i',$params,$match)){
            $data['cols'] = $match[1];
        }else{
            $data['cols'] = 5;
        }

        //show the filename
        if(preg_match('/\bshowname\b/i',$params,$match)) {
            $data['showname'] = true;
        }else{
            $data['showname'] = false;
        }

        return $data;
    }

    /**
     * Create output
     */
    function render($mode, &$renderer, $data) {
        if($mode == 'xhtml'){
	    global $ID;
	    $meta= array();
	    $meta = p_get_metadata($ID);

	    $data['desc']= $meta['desc'];
	    $data['screenshot']= $meta['screenshot'];
	    $data['logo']= $meta['logo'];
            $data['topic']= $meta['topic'];
            $data['version']= $meta['version'];
            $data['download']= $meta['download'];
            $data['link']= $meta['link'];
            $data['banner']= $meta['banner'];
            $data['author']= $meta['author'];
            $data['title']= $meta['title'];

            $data['example']= $meta['example'];
            $data['manual']= $meta['manual'];
            $data['forum']= $meta['forum'];
	# more meta-keys: 
        # example, manual, forum, tutorial
        # rpm, deb info (eg. http://packages.debian.org/unstable/sound/ardour-gtk )
        # ? Install, Prerequisites
        # ? mailinglist, support
	# ? distribution, OS
        # ? configfile, config
 

	 # TODO: collect unknown meta tags 
         # include key->data listing in folded sub-sub-page

	    if ( $data['title'] == "") $data['title']= $ID;

            $renderer->doc .= $this->_vcard($data);
            return true;
	}
        return false;
    }

    /**
     * Does the vcard formatting and rendering
     */
    function _vcard($data){
        global $conf;
        global $lang;
        $ret = '';
        $ret .= '<table class="vcard">';
	if ($data[logo]) 
		$ret .= '<tr><td style="text-align:center;">'.$this->_image($data[logo],"Logo",64).'</td><th style="text-align:center;">'.$data[title].'</th></tr>';
	else 
		$ret .= '<tr><th colspan="2" style="text-align:center;">'.$data[title].'</th></tr>';
	if ($data[topic]) 
		$ret .= '<tr><th>Topic:</th><td>'.$this->_ulink("apps:categories:".$data[topic],$data[topic]).'</td></tr>';
	if ($data[link]) {
		if (strlen($data[link]) > 32) 
			$ret .= '<tr><th>Website:</th><td><a href="'.$data[link].'" target="_blank" title="'.$data[link].'">'.substr($data[link],0,31).'...</a></td></tr>';
		else	$ret .= '<tr><th>Website:</th><td><a href="'.$data[link].'" target="_blank" title="'.$data[link].'">'.$data[link].'</a></td></tr>';
	}

        $ret .= '<tr><td colspan="2" class="desc">'.$data[desc].'</td></tr>';

	if ($data[banner]) 
		$ret .= '<tr><td colspan="2" style="text-align:center;">Banner:<br/>'.$this->_image($data[banner],"Banner",220).'</td></tr>';
	if ($data[screenshot]) 
		$ret .= '<tr><td colspan="2" style="text-align:center;">Screenshot:<br/>'.$this->_image($data[screenshot],"Screenshot",220).'</td></tr>';
	if ($data[author]) 
		$ret .= '<tr><th>Author:</th><td>'.$data[author].'</td></tr>';
# TODO: group download = version +
	if ($data[version]) 
		$ret .= '<tr><th>Version:</th><td>'.$data[version].'</td></tr>';
	if ($data[download]) 
		$ret .= '<tr><th>Download:</th><td>'.$this->_ulink($data[download],$data[download]).'</td></tr>';
	if ($data[forum]) 
		$ret .= '<tr><th>Forum:</th><td>'.$this->_ulink($data[forum],$data[forum]).'</td></tr>';
	if ($data[manual]) 
		$ret .= '<tr><th>Manual:</th><td>'.$this->_ulink($data[manual],$data[manual]).'</td></tr>';
	if ($data[example]) 
		$ret .= '<tr><th>Example:</th><td>'.$this->_ulink($data[example],$data[example]).'</td></tr>';
        $ret .= '</table>';
        return $ret;
    }

    /**
     * Defines how a image should look like
     */
    function _image($img,$alt,$w){

        $src = ml($img,array('w'=>$w));
        $lnk = ml($img,array(),false);

        //prepare attributes
        $p           = array();
        $p['width']  = $w;
        $p['border'] = 0;
        $p['alt']  = $alt;
        $p['title']  = $alt;
        $p['class']  = 'tn';
        $att = buildAttributes($p);

        // prepare output
        $ret  = '';
        $ret .= '<a href="'.$lnk.'">';
        $ret .= '<img src="'.$src.'" '.$att.' />';
        $ret .= '</a>';
        return $ret;
    }


    /**
     * Defines how a link should look
     */
    function _ulink($url,$text){
 	if (substr($url,0,7) == "http://" ) {
	  $lnk=$url;
	} else {
          $lnk = wl($url,array(),false);
 	}
        // prepare output
        $ret  = '';
        $ret .= '<a href="'.$lnk.'" title="'.htmlspecialchars($text).'">';
	if (strlen($text) > 32) {
		$text= substr($text, 0, 31)."...";
	}
        $ret .= htmlspecialchars($text);
        $ret .= '</a>';
        return $ret;
    }

}

